//
//  tank
//
//  Import and Playlist Daemon for autoradio project
//
//
//  Copyright (C) 2017-2018 Christian Pointner <equinox@helsinki.at>
//
//  This file is part of tank.
//
//  tank is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  any later version.
//
//  tank is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with tank. If not, see <http://www.gnu.org/licenses/>.
//

package importer

import (
	"io"
	"time"
)

//******* null

type JobSourceNull uint64

func newJobSourceNull(len uint64) *JobSourceNull {
	src := JobSourceNull(len)
	return &src
}

func (src *JobSourceNull) Len() uint64 {
	return uint64(*src)
}

func (src *JobSourceNull) Read(p []byte) (n int, err error) {
	l := len(p)
	if *src > JobSourceNull(l) {
		*src = *src - JobSourceNull(l)
		time.Sleep(20 * time.Millisecond)
		return l, nil
	}
	l = int(*src)
	*src = 0
	return l, io.EOF
}

func (src *JobSourceNull) Done(result *JobSourceResult) {
	return
}

//******* Attachment

type JobSourceAttachment struct {
	len  uint64
	r    io.Reader
	done chan *JobSourceResult
}

func newJobSourceAttachment(len uint64, r io.Reader) *JobSourceAttachment {
	src := &JobSourceAttachment{len: len, r: r}
	src.done = make(chan *JobSourceResult, 1)
	return src
}

func (src *JobSourceAttachment) Len() uint64 {
	return src.len
}

func (src *JobSourceAttachment) Read(p []byte) (n int, err error) {
	return src.r.Read(p)
}

func (src *JobSourceAttachment) Done(result *JobSourceResult) {
	if result == nil {
		close(src.done)
		return
	}
	src.done <- result
}
